@@ -55,7 +55,8 @@ void g_isi_sb_iter_init_full(GIsiSubBlockIter *iter, const GIsiMessage *msg,
 	if (data == NULL)
 		len = used = 0;
 
-	iter->start = (uint8_t *)data + used;
+	iter->cursor = longhdr ? 4 : 2;
+	iter->start = (uint8_t *) data + used;
 	iter->end = iter->start + len;
 	iter->longhdr = longhdr;
 	iter->sub_blocks = len > used ? sub_blocks : 0;
@@ -70,7 +71,8 @@ void g_isi_sb_iter_init(GIsiSubBlockIter *iter, const GIsiMessage *msg,
 	if (data == NULL)
 		len = used = 0;
 
-	iter->start = (uint8_t *)data + used;
+	iter->cursor = 2;
+	iter->start = (uint8_t *) data + used;
 	iter->end = iter->start + len;
 	iter->longhdr = FALSE;
 	iter->sub_blocks = len > used ? iter->start[-1] : 0;
@@ -85,6 +87,7 @@ void g_isi_sb_subiter_init(GIsiSubBlockIter *outer, GIsiSubBlockIter *inner,
 			outer->start + used > outer->end)
 		len = used = 0;
 
+	inner->cursor = 2;
 	inner->start = outer->start + used;
 	inner->end = inner->start + len;
 	inner->longhdr = FALSE;
@@ -101,6 +104,7 @@ void g_isi_sb_subiter_init_full(GIsiSubBlockIter *outer,
 			outer->start + used > outer->end)
 		len = used = 0;
 
+	inner->cursor = longhdr ? 4 : 2;
 	inner->start = outer->start + used;
 	inner->end = inner->start + len;
 	inner->longhdr = longhdr;
@@ -143,18 +147,18 @@ size_t g_isi_sb_iter_get_len(const GIsiSubBlockIter *iter)
 gboolean g_isi_sb_iter_get_data(const GIsiSubBlockIter *restrict iter,
 				void **data, unsigned pos)
 {
-	if ((size_t)pos > g_isi_sb_iter_get_len(iter)
+	if ((size_t) pos > g_isi_sb_iter_get_len(iter)
 			|| iter->start + pos > iter->end)
 		return FALSE;
 
-	*data = (void *)iter->start + pos;
+	*data = (void *) iter->start + pos;
 	return TRUE;
 }
 
 gboolean g_isi_sb_iter_get_byte(const GIsiSubBlockIter *restrict iter,
 				uint8_t *byte, unsigned pos)
 {
-	if ((size_t)pos > g_isi_sb_iter_get_len(iter)
+	if ((size_t) pos > g_isi_sb_iter_get_len(iter)
 			|| iter->start + pos > iter->end)
 		return FALSE;
 
@@ -188,6 +192,35 @@ gboolean g_isi_sb_iter_get_dword(const GIsiSubBlockIter *restrict iter,
 	return TRUE;
 }
 
+gboolean g_isi_sb_iter_eat_byte(GIsiSubBlockIter *restrict iter,
+				uint8_t *byte)
+{
+	if (!g_isi_sb_iter_get_byte(iter, byte, iter->cursor))
+		return FALSE;
+
+	iter->cursor += 1;
+	return TRUE;
+}
+gboolean g_isi_sb_iter_eat_word(GIsiSubBlockIter *restrict iter,
+				uint16_t *word)
+{
+	if (!g_isi_sb_iter_get_word(iter, word, iter->cursor))
+		return FALSE;
+
+	iter->cursor += 2;
+	return TRUE;
+}
+
+gboolean g_isi_sb_iter_eat_dword(GIsiSubBlockIter *restrict iter,
+				uint32_t *dword)
+{
+	if (!g_isi_sb_iter_get_dword(iter, dword, iter->cursor))
+		return FALSE;
+
+	iter->cursor += 4;
+	return TRUE;
+}
+
 gboolean g_isi_sb_iter_get_oper_code(const GIsiSubBlockIter *restrict iter,
 					char *mcc, char *mnc, unsigned pos)
 {
@@ -198,6 +231,16 @@ gboolean g_isi_sb_iter_get_oper_code(const GIsiSubBlockIter *restrict iter,
 	return TRUE;
 }
 
+gboolean g_isi_sb_iter_eat_oper_code(GIsiSubBlockIter *restrict iter,
+					char *mcc, char *mnc)
+{
+	if (!g_isi_sb_iter_get_oper_code(iter, mcc, mnc, iter->cursor))
+		return FALSE;
+
+	iter->cursor += 3;
+	return TRUE;
+}
+
 gboolean g_isi_sb_iter_get_alpha_tag(const GIsiSubBlockIter *restrict iter,
 					char **utf8, size_t len, unsigned pos)
 {
@@ -214,11 +257,20 @@ gboolean g_isi_sb_iter_get_alpha_tag(const GIsiSubBlockIter *restrict iter,
 	if (ucs2 + len > iter->end)
 		return FALSE;
 
-	*utf8 = g_convert((const char *)ucs2, len, "UTF-8//TRANSLIT", "UCS-2BE",
-				NULL, NULL, NULL);
+	*utf8 = g_convert((const char *) ucs2, len, "UTF-8//TRANSLIT",
+				"UCS-2BE", NULL, NULL, NULL);
 	return *utf8 != NULL;
 }
 
+gboolean g_isi_sb_iter_eat_alpha_tag(GIsiSubBlockIter *restrict iter,
+					char **utf8, size_t len)
+{
+	if (!g_isi_sb_iter_get_alpha_tag(iter, utf8, len, iter->cursor))
+		return FALSE;
+
+	iter->cursor += len;
+	return TRUE;
+}
 gboolean g_isi_sb_iter_get_latin_tag(const GIsiSubBlockIter *restrict iter,
 					char **latin, size_t len, unsigned pos)
 {
@@ -238,11 +290,20 @@ gboolean g_isi_sb_iter_get_latin_tag(const GIsiSubBlockIter *restrict iter,
 	if (str + len > iter->end)
 		return FALSE;
 
-	*latin = g_strndup((char *)str, len);
+	*latin = g_strndup((char *) str, len);
 
 	return *latin != NULL;
 }
 
+gboolean g_isi_sb_iter_eat_latin_tag(GIsiSubBlockIter *restrict iter,
+					char **latin, size_t len)
+{
+	if (!g_isi_sb_iter_get_latin_tag(iter, latin, len, iter->cursor))
+		return FALSE;
+
+	iter->cursor += len;
+	return TRUE;
+}
 gboolean g_isi_sb_iter_next(GIsiSubBlockIter *iter)
 {
 	uint8_t len = g_isi_sb_iter_get_len(iter);
@@ -256,6 +317,8 @@ gboolean g_isi_sb_iter_next(GIsiSubBlockIter *iter)
 	if (iter->start + len > iter->end)
 		return FALSE;
 
+
+	iter->cursor = iter->longhdr ? 4 : 2;
 	iter->start += len;
 	iter->sub_blocks--;
 
