@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
 
@@ -38,11 +37,8 @@
 #include <ofono/devinfo.h>
 #include <ofono/netreg.h>
 #include <ofono/sim.h>
-#include <ofono/cbs.h>
-#include <ofono/sms.h>
-#include <ofono/ussd.h>
 #include <ofono/gprs.h>
-#include <ofono/phonebook.h>
+#include <ofono/gprs-context.h>
 
 #include <drivers/atmodem/atutil.h>
 #include <drivers/atmodem/vendor.h>
@@ -53,6 +49,7 @@ struct gobi_data {
 	GAtChat *chat;
 	struct ofono_sim *sim;
 	gboolean have_sim;
+	guint cfun_timeout;
 };
 
 static void gobi_debug(const char *str, void *user_data)
@@ -92,8 +89,8 @@ static GAtChat *open_device(struct ofono_modem *modem,
 				const char *key, char *debug)
 {
 	const char *device;
-	GAtSyntax *syntax;
 	GIOChannel *channel;
+	GAtSyntax *syntax;
 	GAtChat *chat;
 
 	device = ofono_modem_get_string(modem, key);
@@ -109,6 +106,7 @@ static GAtChat *open_device(struct ofono_modem *modem,
 	syntax = g_at_syntax_new_gsm_permissive();
 	chat = g_at_chat_new(channel, syntax);
 	g_at_syntax_unref(syntax);
+
 	g_io_channel_unref(channel);
 
 	if (chat == NULL)
@@ -124,7 +122,6 @@ static void simstat_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
 	struct gobi_data *data = ofono_modem_get_data(modem);
-
 	GAtResultIter iter;
 	const char *state, *tmp;
 
@@ -166,9 +163,20 @@ static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
 
 	DBG("");
 
-	data->have_sim = FALSE;
+	if (data->cfun_timeout > 0) {
+		g_source_remove(data->cfun_timeout);
+		data->cfun_timeout = 0;
+	}
 
-	ofono_modem_set_powered(modem, ok);
+	if (!ok) {
+		g_at_chat_unref(data->chat);
+		data->chat = NULL;
+
+		ofono_modem_set_powered(modem, FALSE);
+		return;
+	}
+
+	data->have_sim = FALSE;
 
 	g_at_chat_register(data->chat, "$QCSIMSTAT:", simstat_notify,
 						FALSE, modem, NULL);
@@ -178,6 +186,25 @@ static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
 
 	g_at_chat_send(data->chat, "AT$QCSIMSTAT?", none_prefix,
 						NULL, NULL, NULL);
+
+	ofono_modem_set_powered(modem, TRUE);
+}
+
+static gboolean cfun_timeout(gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct gobi_data *data = ofono_modem_get_data(modem);
+
+	ofono_error("Modem enabling timeout, RFKILL enabled?");
+
+	data->cfun_timeout = 0;
+
+	g_at_chat_unref(data->chat);
+	data->chat = NULL;
+
+	ofono_modem_set_powered(modem, FALSE);
+
+	return FALSE;
 }
 
 static int gobi_enable(struct ofono_modem *modem)
@@ -190,12 +217,13 @@ static int gobi_enable(struct ofono_modem *modem)
 	if (data->chat == NULL)
 		return -EINVAL;
 
-	g_at_chat_send(data->chat, "ATE0 +CMEE=1", none_prefix,
-						NULL, NULL, NULL);
+	g_at_chat_send(data->chat, "ATE0 +CMEE=1", NULL, NULL, NULL, NULL);
 
 	g_at_chat_send(data->chat, "AT+CFUN=4", none_prefix,
 					cfun_enable, modem, NULL);
 
+	data->cfun_timeout = g_timeout_add_seconds(5, cfun_timeout, modem);
+
 	return -EINPROGRESS;
 }
 
@@ -219,9 +247,6 @@ static int gobi_disable(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
-	if (data->chat == NULL)
-		return 0;
-
 	g_at_chat_cancel_all(data->chat);
 	g_at_chat_unregister_all(data->chat);
 
@@ -235,11 +260,10 @@ static void set_online_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	ofono_modem_online_cb_t cb = cbd->cb;
+	struct ofono_error error;
 
-	if (ok)
-		CALLBACK_WITH_SUCCESS(cb, cbd->data);
-	else
-		CALLBACK_WITH_FAILURE(cb, cbd->data);
+	decode_at_error(&error, g_at_result_final_response(result));
+	cb(&error, cbd->data);
 }
 
 static void gobi_set_online(struct ofono_modem *modem, ofono_bool_t online,
@@ -251,17 +275,13 @@ static void gobi_set_online(struct ofono_modem *modem, ofono_bool_t online,
 
 	DBG("modem %p %s", modem, online ? "online" : "offline");
 
-	if (data->chat == NULL)
-		goto error;
-
-	if (g_at_chat_send(data->chat, command, NULL,
-					set_online_cb, cbd, g_free))
+	if (g_at_chat_send(data->chat, command, none_prefix,
+					set_online_cb, cbd, g_free) > 0)
 		return;
 
-error:
-	g_free(cbd);
-
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
+
+	g_free(cbd);
 }
 
 static void gobi_pre_sim(struct ofono_modem *modem)
@@ -277,28 +297,26 @@ static void gobi_pre_sim(struct ofono_modem *modem)
 static void gobi_post_sim(struct ofono_modem *modem)
 {
 	struct gobi_data *data = ofono_modem_get_data(modem);
+	struct ofono_gprs *gprs;
+	struct ofono_gprs_context *gc;
 
 	DBG("%p", modem);
 
-	ofono_phonebook_create(modem, 0, "atmodem", data->chat);
+	gprs = ofono_gprs_create(modem, OFONO_VENDOR_GOBI,
+						"atmodem", data->chat);
+	gc = ofono_gprs_context_create(modem, 0, "atmodem", data->chat);
 
-	ofono_sms_create(modem, OFONO_VENDOR_GOBI, "atmodem", data->chat);
+	if (gprs && gc)
+		ofono_gprs_add_context(gprs, gc);
 }
 
 static void gobi_post_online(struct ofono_modem *modem)
 {
 	struct gobi_data *data = ofono_modem_get_data(modem);
-	struct ofono_gprs *gprs;
 
 	DBG("%p", modem);
 
-	ofono_netreg_create(modem, OFONO_VENDOR_GOBI, "atmodem", data->chat);
-
-	ofono_cbs_create(modem, OFONO_VENDOR_GOBI, "atmodem", data->chat);
-	ofono_ussd_create(modem, OFONO_VENDOR_GOBI, "atmodem", data->chat);
-
-	gprs = ofono_gprs_create(modem, OFONO_VENDOR_GOBI,
-						"atmodem", data->chat);
+	ofono_netreg_create(modem, 0, "dunmodem", data->chat);
 }
 
 static struct ofono_modem_driver gobi_driver = {
@@ -307,10 +325,10 @@ static struct ofono_modem_driver gobi_driver = {
 	.remove		= gobi_remove,
 	.enable		= gobi_enable,
 	.disable	= gobi_disable,
-	.set_online     = gobi_set_online,
+	.set_online	= gobi_set_online,
 	.pre_sim	= gobi_pre_sim,
 	.post_sim	= gobi_post_sim,
-	.post_online    = gobi_post_online,
+	.post_online	= gobi_post_online,
 };
 
 static int gobi_init(void)
