@@ -63,6 +63,9 @@
 #include <drivers/atmodem/vendor.h>
 #include <drivers/atmodem/sim-poll.h>
 #include <drivers/atmodem/atutil.h>
+#include <drivers/hfpmodem/slc.h>
+
+#include "ofono.h"
 
 static const char *none_prefix[] = { NULL };
 static const char *ptty_prefix[] = { "+PTTY:", NULL };
@@ -73,6 +76,10 @@ struct phonesim_data {
 	GAtChat *chat;
 	gboolean calypso;
 	gboolean use_mux;
+	gboolean hfp;
+	struct hfp_slc_info hfp_info;
+	unsigned int hfp_watch;
+	int batt_level;
 };
 
 struct gprs_context_data {
@@ -381,9 +388,9 @@ static void phonesim_remove(struct ofono_modem *modem)
 	ofono_modem_set_data(modem, NULL);
 }
 
-static void phonesim_debug(const char *str, void *user_data)
+static void phonesim_debug(const char *str, void *prefix)
 {
-	ofono_info("%s", str);
+	ofono_info("%s%s", (const char *) prefix, str);
 }
 
 static void cfun_set_on_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -419,6 +426,43 @@ static void crst_notify(GAtResult *result, gpointer user_data)
 	g_idle_add(phonesim_reset, user_data);
 }
 
+static void emulator_battery_cb(struct ofono_atom *atom, void *data)
+{
+	struct ofono_emulator *em = __ofono_atom_get_data(atom);
+	int val = 0;
+
+	if (GPOINTER_TO_INT(data) > 0)
+		val = (GPOINTER_TO_INT(data) - 1) / 20 + 1;
+
+	ofono_emulator_set_indicator(em, OFONO_EMULATOR_IND_BATTERY, val);
+}
+
+static void cbc_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct phonesim_data *data = ofono_modem_get_data(modem);
+	GAtResultIter iter;
+	int status;
+	int level;
+
+	g_at_result_iter_init(&iter, result);
+	if (!g_at_result_iter_next(&iter, "+CBC:"))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &status))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter, &level))
+		return;
+
+	data->batt_level = level;
+
+	__ofono_modem_foreach_registered_atom(modem,
+						OFONO_ATOM_TYPE_EMULATOR_HFP,
+						emulator_battery_cb,
+						GUINT_TO_POINTER(level));
+}
+
 static void phonesim_disconnected(gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
@@ -455,7 +499,7 @@ static void mux_setup(GAtMux *mux, gpointer user_data)
 	data->mux = mux;
 
 	if (getenv("OFONO_AT_DEBUG"))
-		g_at_mux_set_debug(data->mux, phonesim_debug, NULL);
+		g_at_mux_set_debug(data->mux, phonesim_debug, "");
 
 	g_at_mux_start(mux);
 	io = g_at_mux_create_channel(mux);
@@ -470,7 +514,7 @@ static void mux_setup(GAtMux *mux, gpointer user_data)
 	g_io_channel_unref(io);
 
 	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(data->chat, phonesim_debug, NULL);
+		g_at_chat_set_debug(data->chat, phonesim_debug, "");
 
 	if (data->calypso)
 		g_at_chat_set_wakeup_command(data->chat, "AT\r", 500, 5000);
@@ -481,14 +525,49 @@ static void mux_setup(GAtMux *mux, gpointer user_data)
 					cfun_set_on_cb, modem, NULL);
 }
 
+static void emulator_hfp_watch(struct ofono_atom *atom,
+				enum ofono_atom_watch_condition cond,
+				void *user_data)
+{
+	struct phonesim_data *data = user_data;
+
+	if (cond != OFONO_ATOM_WATCH_CONDITION_REGISTERED)
+		return;
+
+	emulator_battery_cb(atom, GUINT_TO_POINTER(data->batt_level));
+}
+
+static int connect_socket(const char *address, int port)
+{
+	struct sockaddr_in addr;
+	int sk;
+	int err;
+
+	sk = socket(PF_INET, SOCK_STREAM, 0);
+	if (sk < 0)
+		return -EINVAL;
+
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = inet_addr(address);
+	addr.sin_port = htons(port);
+
+	err = connect(sk, (struct sockaddr *) &addr, sizeof(addr));
+	if (err < 0) {
+		close(sk);
+		return -errno;
+	}
+
+	return sk;
+}
+
 static int phonesim_enable(struct ofono_modem *modem)
 {
 	struct phonesim_data *data = ofono_modem_get_data(modem);
 	GIOChannel *io;
 	GAtSyntax *syntax;
-	struct sockaddr_in addr;
 	const char *address, *value;
-	int sk, err, port;
+	int sk, port;
 
 	DBG("%p", modem);
 
@@ -508,20 +587,9 @@ static int phonesim_enable(struct ofono_modem *modem)
 	if (!g_strcmp0(value, "internal"))
 		data->use_mux = TRUE;
 
-	sk = socket(PF_INET, SOCK_STREAM, 0);
+	sk = connect_socket(address, port);
 	if (sk < 0)
-		return -EINVAL;
-
-	memset(&addr, 0, sizeof(addr));
-	addr.sin_family = AF_INET;
-	addr.sin_addr.s_addr = inet_addr(address);
-	addr.sin_port = htons(port);
-
-	err = connect(sk, (struct sockaddr *) &addr, sizeof(addr));
-	if (err < 0) {
-		close(sk);
-		return err;
-	}
+		return sk;
 
 	io = g_io_channel_unix_new(sk);
 	if (io == NULL) {
@@ -543,7 +611,7 @@ static int phonesim_enable(struct ofono_modem *modem)
 		return -ENOMEM;
 
 	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(data->chat, phonesim_debug, NULL);
+		g_at_chat_set_debug(data->chat, phonesim_debug, "");
 
 	g_at_chat_set_disconnect_function(data->chat,
 						phonesim_disconnected, modem);
@@ -574,6 +642,15 @@ static int phonesim_enable(struct ofono_modem *modem)
 	g_at_chat_register(data->chat, "+CRST:",
 				crst_notify, FALSE, modem, NULL);
 
+	g_at_chat_register(data->chat, "+CBC:",
+				cbc_notify, FALSE, modem, NULL);
+
+	g_at_chat_send(data->chat, "AT+CBC", none_prefix, NULL, NULL, NULL);
+
+	data->hfp_watch = __ofono_modem_add_atom_watch(modem,
+					OFONO_ATOM_TYPE_EMULATOR_HFP,
+					emulator_hfp_watch, data, NULL);
+
 	return 0;
 }
 
@@ -612,6 +689,8 @@ static int phonesim_disable(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
+	__ofono_modem_remove_atom_watch(modem, data->hfp_watch);
+
 	g_at_chat_unref(data->chat);
 	data->chat = NULL;
 
@@ -718,14 +797,142 @@ static struct ofono_modem_driver phonesim_driver = {
 	.post_online	= phonesim_post_online,
 };
 
+static int localhfp_probe(struct ofono_modem *modem)
+{
+	struct hfp_slc_info *info;
+
+	DBG("%p", modem);
+
+	info = g_try_new(struct hfp_slc_info, 1);
+	if (info == NULL)
+		return -ENOMEM;
+
+	ofono_modem_set_data(modem, info);
+
+	return 0;
+}
+
+static void localhfp_remove(struct ofono_modem *modem)
+{
+	struct hfp_slc_info *info = ofono_modem_get_data(modem);
+
+	DBG("%p", modem);
+
+	g_free(info);
+	ofono_modem_set_data(modem, NULL);
+}
+
+static void slc_established(gpointer userdata)
+{
+	struct ofono_modem *modem = userdata;
+
+	ofono_modem_set_powered(modem, TRUE);
+}
+
+static void slc_failed(gpointer userdata)
+{
+	struct ofono_modem *modem = userdata;
+	struct hfp_slc_info *info = ofono_modem_get_data(modem);
+
+	ofono_modem_set_powered(modem, FALSE);
+
+	g_at_chat_unref(info->chat);
+	info->chat = NULL;
+}
+
+static int localhfp_enable(struct ofono_modem *modem)
+{
+	struct hfp_slc_info *info = ofono_modem_get_data(modem);
+	GIOChannel *io;
+	GAtSyntax *syntax;
+	GAtChat *chat;
+	const char *address;
+	int sk, port;
+
+	address = ofono_modem_get_string(modem, "Address");
+	if (address == NULL)
+		return -EINVAL;
+
+	port = ofono_modem_get_integer(modem, "Port");
+	if (port < 0)
+		return -EINVAL;
+
+	sk = connect_socket(address, port);
+	if (sk < 0)
+		return sk;
+
+	io = g_io_channel_unix_new(sk);
+	if (io == NULL) {
+		close(sk);
+		return -ENOMEM;
+	}
+
+	syntax = g_at_syntax_new_gsmv1();
+	chat = g_at_chat_new(io, syntax);
+	g_at_syntax_unref(syntax);
+	g_io_channel_unref(io);
+
+	if (chat == NULL)
+		return -ENOMEM;
+
+	if (getenv("OFONO_AT_DEBUG"))
+		g_at_chat_set_debug(chat, phonesim_debug, "LocalHfp: ");
+
+	g_at_chat_set_disconnect_function(chat, slc_failed, modem);
+
+	hfp_slc_info_init(info);
+	info->chat = chat;
+	hfp_slc_establish(info, slc_established, slc_failed, modem);
+
+	return -EINPROGRESS;
+}
+
+static int localhfp_disable(struct ofono_modem *modem)
+{
+	struct hfp_slc_info *info = ofono_modem_get_data(modem);
+
+	g_at_chat_unref(info->chat);
+	info->chat = NULL;
+
+	return 0;
+}
+
+static void localhfp_pre_sim(struct ofono_modem *modem)
+{
+	struct hfp_slc_info *info = ofono_modem_get_data(modem);
+
+	DBG("%p", modem);
+
+	ofono_voicecall_create(modem, 0, "hfpmodem", info);
+	ofono_netreg_create(modem, 0, "hfpmodem", info);
+	ofono_call_volume_create(modem, 0, "hfpmodem", info);
+}
+
+static struct ofono_modem_driver localhfp_driver = {
+	.name		= "localhfp",
+	.probe		= localhfp_probe,
+	.remove		= localhfp_remove,
+	.enable		= localhfp_enable,
+	.disable	= localhfp_disable,
+	.pre_sim	= localhfp_pre_sim,
+};
+
 static struct ofono_modem *create_modem(GKeyFile *keyfile, const char *group)
 {
+	const char *driver = "phonesim";
 	struct ofono_modem *modem;
 	char *value;
 
 	DBG("group %s", group);
 
-	modem = ofono_modem_create(group, "phonesim");
+	value = g_key_file_get_string(keyfile, group, "Modem", NULL);
+
+	if (value && g_str_equal(value, "hfp"))
+		driver = "localhfp";
+
+	g_free(value);
+
+	modem = ofono_modem_create(group, driver);
 	if (modem == NULL)
 		return NULL;
 
@@ -816,8 +1023,9 @@ static int phonesim_init(void)
 	if (err < 0)
 		return err;
 
-	ofono_gprs_context_driver_register(&context_driver);
+	ofono_modem_driver_register(&localhfp_driver);
 
+	ofono_gprs_context_driver_register(&context_driver);
 	ofono_ctm_driver_register(&ctm_driver);
 
 	parse_config(CONFIGDIR "/phonesim.conf");
