@@ -116,39 +116,49 @@ static void add_mbm(struct ofono_modem *modem,
 	DBG("desc: %s", desc);
 
 	registered = ofono_modem_get_integer(modem, "Registered");
-	if (registered != 0)
-		return;
 
-	if (g_str_has_suffix(desc, "Minicard Modem") ||
+	if (registered == 0 &&
+			(g_str_has_suffix(desc, "Minicard Modem") ||
 			g_str_has_suffix(desc, "Minicard Modem 2") ||
 			g_str_has_suffix(desc, "Mini-Card Modem") ||
 			g_str_has_suffix(desc, "Broadband Modem") ||
-			g_str_has_suffix(desc, "Broadband USB Modem")) {
+			g_str_has_suffix(desc, "Module Modem") ||
+			g_str_has_suffix(desc, "Broadband USB Modem"))) {
 		devnode = udev_device_get_devnode(udev_device);
+
 		if (ofono_modem_get_string(modem, MODEM_DEVICE) == NULL)
 			ofono_modem_set_string(modem, MODEM_DEVICE, devnode);
 		else
 			ofono_modem_set_string(modem, DATA_DEVICE, devnode);
-	} else if (g_str_has_suffix(desc, "Minicard Data Modem") ||
+	} else if (registered == 0 &&
+			(g_str_has_suffix(desc, "Minicard Data Modem") ||
 			g_str_has_suffix(desc, "Mini-Card Data Modem") ||
-			g_str_has_suffix(desc, "Broadband Data Modem")) {
+			g_str_has_suffix(desc, "Module Data Modem") ||
+			g_str_has_suffix(desc, "Module\xc2\xa0""Data Modem") ||
+			g_str_has_suffix(desc, "Broadband Data Modem"))) {
 		devnode = udev_device_get_devnode(udev_device);
 		ofono_modem_set_string(modem, DATA_DEVICE, devnode);
 	} else if (g_str_has_suffix(desc, "Minicard GPS Port") ||
 			g_str_has_suffix(desc, "Mini-Card GPS Port") ||
+			g_str_has_suffix(desc, "Module NMEA") ||
 			g_str_has_suffix(desc, "Broadband GPS Port")) {
 		devnode = udev_device_get_devnode(udev_device);
 		ofono_modem_set_string(modem, GPS_DEVICE, devnode);
-	} else if (g_str_has_suffix(desc, "Minicard Network Adapter") ||
+	} else if (registered == 0 &&
+			(g_str_has_suffix(desc, "Minicard Network Adapter") ||
 			g_str_has_suffix(desc, "Mini-Card Network Adapter") ||
 			g_str_has_suffix(desc, "Broadband Network Adapter") ||
-			g_str_has_suffix(desc, "Minicard NetworkAdapter")) {
+			g_str_has_suffix(desc, "Module Network Adapter") ||
+			g_str_has_suffix(desc, "Minicard NetworkAdapter"))) {
 		devnode = get_property(udev_device, "INTERFACE");
 		ofono_modem_set_string(modem, NETWORK_INTERFACE, devnode);
 	} else {
 		return;
 	}
 
+	if (registered == 1)
+		return;
+
 	device  = ofono_modem_get_string(modem, MODEM_DEVICE);
 	data = ofono_modem_get_string(modem, DATA_DEVICE);
 	network = ofono_modem_get_string(modem, NETWORK_INTERFACE);
@@ -290,7 +300,7 @@ static void add_huawei(struct ofono_modem *modem,
 					struct udev_device *udev_device)
 {
 	struct udev_list_entry *entry;
-	const char *devnode, *type;
+	const char *devnode;
 	int ppp, pcui;
 
 	DBG("modem %p", modem);
@@ -315,15 +325,7 @@ static void add_huawei(struct ofono_modem *modem,
 	entry = udev_device_get_properties_list_entry(udev_device);
 	while (entry) {
 		const char *name = udev_list_entry_get_name(entry);
-		type = udev_list_entry_get_value(entry);
-
-		if (g_str_equal(name, "OFONO_HUAWEI_VOICE") == TRUE) {
-			gboolean value = g_str_equal(type, "1");
-
-			ofono_modem_set_boolean(modem, "HasVoice", value);
-			entry = udev_list_entry_get_next(entry);
-			continue;
-		}
+		const char *type = udev_list_entry_get_value(entry);
 
 		if (g_str_equal(name, "OFONO_HUAWEI_TYPE") != TRUE) {
 			entry = udev_list_entry_get_next(entry);
@@ -347,9 +349,6 @@ static void add_huawei(struct ofono_modem *modem,
 
 			pcui = 1;
 			ofono_modem_set_integer(modem, "PcuiRegistered", pcui);
-		} else if (g_str_equal(type, "NDIS") == TRUE) {
-			devnode = udev_device_get_devnode(udev_device);
-			ofono_modem_set_string(modem, "NDIS", devnode);
 		}
 
 		break;
@@ -395,38 +394,39 @@ static void add_novatel(struct ofono_modem *modem,
 					struct udev_device *udev_device)
 {
 	const char *devnode, *intfnum;
-	struct udev_device *parent;
 	int registered;
 
 	DBG("modem %p", modem);
 
 	registered = ofono_modem_get_integer(modem, "Registered");
-	if (registered != 0)
-		return;
 
-	parent = udev_device_get_parent(udev_device);
-	parent = udev_device_get_parent(parent);
-	intfnum = udev_device_get_sysattr_value(parent, "bInterfaceNumber");
+        if (registered > 1)
+                return;
+
+	intfnum = get_property(udev_device, "ID_USB_INTERFACE_NUM");
 
 	DBG("intfnum %s", intfnum);
 
 	if (g_strcmp0(intfnum, "00") == 0) {
 		devnode = udev_device_get_devnode(udev_device);
 		ofono_modem_set_string(modem, "PrimaryDevice", devnode);
+
+		ofono_modem_set_integer(modem, "Registered", ++registered);
 	} else if (g_strcmp0(intfnum, "01") == 0) {
 		devnode = udev_device_get_devnode(udev_device);
 		ofono_modem_set_string(modem, "SecondaryDevice", devnode);
 
-		ofono_modem_set_integer(modem, "Registered", 1);
-		ofono_modem_register(modem);
+		ofono_modem_set_integer(modem, "Registered", ++registered);
 	}
+
+	if (registered > 1)
+		ofono_modem_register(modem);
 }
 
 static void add_nokia(struct ofono_modem *modem,
 					struct udev_device *udev_device)
 {
 	const char *devnode, *intfnum;
-	struct udev_device *parent;
 	int registered;
 
 	DBG("modem %p", modem);
@@ -435,15 +435,14 @@ static void add_nokia(struct ofono_modem *modem,
 	if (registered != 0)
 		return;
 
-	parent = udev_device_get_parent(udev_device);
-	intfnum = udev_device_get_sysattr_value(parent, "bInterfaceNumber");
+	intfnum = get_property(udev_device, "ID_USB_INTERFACE_NUM");
 
 	DBG("intfnum %s", intfnum);
 
-	if (g_strcmp0(intfnum, "01") == 0) {
+	if (g_strcmp0(intfnum, "02") == 0) {
 		devnode = udev_device_get_devnode(udev_device);
 		ofono_modem_set_string(modem, "Modem", devnode);
-	} else if (g_strcmp0(intfnum, "03") == 0) {
+	} else if (g_strcmp0(intfnum, "04") == 0) {
 		devnode = udev_device_get_devnode(udev_device);
 		ofono_modem_set_string(modem, "Control", devnode);
 
@@ -556,7 +555,6 @@ static void add_linktop(struct ofono_modem *modem,
 					struct udev_device *udev_device)
 {
 	const char *devnode, *intfnum;
-	struct udev_device *parent;
 	int registered;
 
 	DBG("modem %p", modem);
@@ -565,23 +563,156 @@ static void add_linktop(struct ofono_modem *modem,
 	if (registered != 0)
 		return;
 
-	parent = udev_device_get_parent(udev_device);
-	intfnum = udev_device_get_sysattr_value(parent, "bInterfaceNumber");
+	intfnum = get_property(udev_device, "ID_USB_INTERFACE_NUM");
 
 	DBG("intfnum %s", intfnum);
 
 	if (g_strcmp0(intfnum, "01") == 0) {
 		devnode = udev_device_get_devnode(udev_device);
-		ofono_modem_set_string(modem, "Modem", devnode);
+		ofono_modem_set_string(modem, "Aux", devnode);
 	} else if (g_strcmp0(intfnum, "03") == 0) {
 		devnode = udev_device_get_devnode(udev_device);
-		ofono_modem_set_string(modem, "Control", devnode);
+		ofono_modem_set_string(modem, "Modem", devnode);
 
 		ofono_modem_set_integer(modem, "Registered", 1);
 		ofono_modem_register(modem);
 	}
 }
 
+static void add_telit(struct ofono_modem *modem,
+					struct udev_device *udev_device)
+{
+	struct udev_list_entry *entry;
+	const char *devnode, *type;
+	int registered;
+
+	DBG("modem %p", modem);
+
+	registered = ofono_modem_get_integer(modem, "Registered");
+
+	entry = udev_device_get_properties_list_entry(udev_device);
+
+	while (entry) {
+		const char *name = udev_list_entry_get_name(entry);
+		type = udev_list_entry_get_value(entry);
+
+		if (g_str_equal(name, "OFONO_TELIT_TYPE") != TRUE) {
+			entry = udev_list_entry_get_next(entry);
+			continue;
+		}
+
+		if (registered == 0 && g_str_equal(type, "Modem") == TRUE) {
+			devnode = udev_device_get_devnode(udev_device);
+			ofono_modem_set_string(modem, "Modem", devnode);
+		} else if (g_str_equal(type, "GPS") == TRUE) {
+			devnode = udev_device_get_devnode(udev_device);
+			ofono_modem_set_string(modem, "GPS", devnode);
+		} else if (registered == 0 &&
+				g_str_equal(type, "Data") == TRUE) {
+			devnode = udev_device_get_devnode(udev_device);
+			ofono_modem_set_string(modem, "Data", devnode);
+		}
+
+		break;
+	}
+
+	if (registered == 1)
+		return;
+
+	if (ofono_modem_get_string(modem, "Modem") != NULL &&
+			ofono_modem_get_string(modem, "Data") != NULL) {
+		ofono_modem_set_integer(modem, "Registered", 1);
+		ofono_modem_register(modem);
+	}
+}
+
+static void add_speedup(struct ofono_modem *modem,
+					struct udev_device *udev_device)
+{
+	struct udev_list_entry *entry;
+	const char *devnode, *type;
+
+	int ppp, aux;
+
+	DBG("modem %p", modem);
+
+	ppp = ofono_modem_get_integer(modem, "ModemRegistered");
+	aux = ofono_modem_get_integer(modem, "AuxRegistered");
+
+	if (ppp && aux)
+		return;
+
+	entry = udev_device_get_properties_list_entry(udev_device);
+	while (entry) {
+		const char *name = udev_list_entry_get_name(entry);
+		type = udev_list_entry_get_value(entry);
+
+		if (g_str_equal(name, "OFONO_SPEEDUP_TYPE") != TRUE) {
+			entry = udev_list_entry_get_next(entry);
+			continue;
+		}
+
+		if (g_str_equal(type, "modem") == TRUE) {
+			if (ppp != 0)
+				return;
+
+			devnode = udev_device_get_devnode(udev_device);
+			ofono_modem_set_string(modem, "Modem", devnode);
+			ppp = 1;
+			ofono_modem_set_integer(modem, "ModemRegistered", ppp);
+		} else if (g_str_equal(type, "aux") == TRUE) {
+			if (aux != 0)
+				return;
+
+			devnode = udev_device_get_devnode(udev_device);
+			ofono_modem_set_string(modem, "Aux", devnode);
+
+			aux = 1;
+			ofono_modem_set_integer(modem, "AuxRegistered", aux);
+		}
+
+		break;
+	}
+
+	if (ppp && aux)
+		ofono_modem_register(modem);
+}
+
+static void add_samsung(struct ofono_modem *modem,
+					struct udev_device *udev_device)
+{
+	const char *subsystem;
+
+	DBG("modem %p", modem);
+
+	subsystem = udev_device_get_subsystem(udev_device);
+	if (subsystem == NULL)
+		return;
+
+	if (g_str_equal(subsystem, "net") == TRUE) {
+		const char *interface;
+
+		interface = get_property(udev_device, "INTERFACE");
+		if (interface == NULL)
+			return;
+
+		DBG("network %s", interface);
+
+		ofono_modem_set_string(modem, "Network", interface);
+	} else if (g_str_equal(subsystem, "tty") == TRUE) {
+		const char *devnode;
+
+		devnode = udev_device_get_devnode(udev_device);
+		if (devnode == NULL)
+			return;
+
+		DBG("device %s", devnode);
+
+		ofono_modem_set_string(modem, "Device", devnode);
+		ofono_modem_register(modem);
+	}
+}
+
 static void add_modem(struct udev_device *udev_device)
 {
 	struct ofono_modem *modem;
@@ -658,6 +789,8 @@ done:
 		add_zte(modem, udev_device);
 	else if (g_strcmp0(driver, "huawei") == 0)
 		add_huawei(modem, udev_device);
+	else if (g_strcmp0(driver, "huaweicdma") == 0)
+		add_huawei(modem, udev_device);
 	else if (g_strcmp0(driver, "sierra") == 0)
 		add_sierra(modem, udev_device);
 	else if (g_strcmp0(driver, "novatel") == 0)
@@ -666,6 +799,8 @@ done:
 		add_nokia(modem, udev_device);
 	else if (g_strcmp0(driver, "isiusb") == 0)
 		add_isi(modem, udev_device);
+	else if (g_strcmp0(driver, "u8500") == 0)
+		add_isi(modem, udev_device);
 	else if (g_strcmp0(driver, "n900") == 0)
 		add_isi(modem, udev_device);
 	else if (g_strcmp0(driver, "gobi") == 0)
@@ -674,10 +809,18 @@ done:
 		add_calypso(modem, udev_device);
 	else if (g_strcmp0(driver, "tc65") == 0)
 		add_tc65(modem, udev_device);
+	else if (g_strcmp0(driver, "telit") == 0)
+		add_telit(modem, udev_device);
 	else if (g_strcmp0(driver, "nokiacdma") == 0)
 		add_nokiacdma(modem, udev_device);
-        else if (g_strcmp0(driver, "linktop") == 0)
+	else if (g_strcmp0(driver, "linktop") == 0)
 		add_linktop(modem, udev_device);
+	else if (g_strcmp0(driver, "speedup") == 0)
+		add_speedup(modem, udev_device);
+	else if (g_strcmp0(driver, "speedupcdma") == 0)
+		add_speedup(modem, udev_device);
+	else if (g_strcmp0(driver, "samsung") == 0)
+		add_samsung(modem, udev_device);
 }
 
 static gboolean devpath_remove(gpointer key, gpointer value, gpointer user_data)
